#define G_LOG_DOMAIN "boilerplate-preferences-view"

#include "config.h"

#include "boilerplate-preferences-view.h"
#include "boilerplate-preferences-page.h"

#include <gtk/gtk.h>

/**
 * SECTION:boilerplate-preferences-view
 * @title: BoilerplatePreferencesView
 * @short_description: A #GtkBin instance created for #BoilerplatePreferences
 * @include: boilerplate-preferences-view.h
 * @see_also: #GtkBin, #BoilerplatePreferencesView
 *
 * The #BoilerplatePreferencesView is an instance #GtkBin, it's the base object
 * for the preferences options. It contains a #GtkStack and manages the adding,
 * removing, and update of options in the preferences view. This allows for the
 * back-end code for managing the preferences ui to be added to either a Window,
 * Dialog or other custom #GtkWidget without re-writing an entire Gobject.
 **/

typedef struct _BoilerplatePreferencesViewPrivate
{
    GtkWidget *box;
    GtkWidget *sidebar;
    GtkWidget *pages;
} BoilerplatePreferencesViewPrivate;

struct _BoilerplatePreferencesView
{
    GtkWindow parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (BoilerplatePreferencesView, boilerplate_preferences_view, GTK_TYPE_BIN)

/**
 * boilerplate_preferences_view_add_page:
 * @self: An instance of #BoilerplatePreferencesView
 * @name: Id of the page for internal usage
 * @title: The title that will apear in the #GtkStackSidebar
 *
 * Adds a new page to the #GtkStack of pages.
 */
void
boilerplate_preferences_view_add_page (BoilerplatePreferencesView *self,
                                       const gchar                *name,
                                       const gchar                *title)
{
    g_return_if_fail (BOILERPLATE_IS_PREFERENCES_VIEW (self));
    g_return_if_fail (name != NULL);
    g_return_if_fail (title != NULL);

    BoilerplatePreferencesViewPrivate *priv = boilerplate_preferences_view_get_instance_private (self);

    BoilerplatePreferencesPage *page = boilerplate_preferences_page_new ();

    gtk_container_add_with_properties (GTK_CONTAINER (priv->pages), GTK_WIDGET (page),
                                       "name", name,
                                       "title", title,
                                       NULL);
}

/**
 * boilerplate_preferences_view_add_page_with_child:
 * @self: An instance of #BoilerplatePreferencesView
 * @name: Id of the page for internal usage
 * @title: The title that will apear in the #GtkStackSidebar
 * @child: A #GtkWidget to add to the new page
 *
 * Adds a child #GtkWidget to a new page, then adds it to #GtkStack of pages.
 */
void
boilerplate_preferences_view_add_page_with_child (BoilerplatePreferencesView *self,
                                                  const gchar                *name,
                                                  const gchar                *title,
                                                  GtkWidget                  *child)
{
    g_return_if_fail (BOILERPLATE_IS_PREFERENCES_VIEW (self));
    g_return_if_fail (name != NULL);
    g_return_if_fail (title != NULL);
    g_return_if_fail (GTK_IS_WIDGET (child));

    GtkWidget *page;

    boilerplate_preferences_view_add_page (self, name, title);

    page = boilerplate_preferences_view_get_page_with_name (self, name);

    gtk_container_add (GTK_CONTAINER (page), GTK_WIDGET (child));
}

/**
 * boilerplate_preferences_view_add_child_to_page:
 * @object: An instance of #BoilerplatePreferencesView
 * @name: Id of the page for internal usage
 * @child: A #GtkWidget to add to the new page
 *
 * Sets a @child into a page with @name.
 */
void
boilerplate_preferences_view_add_child_to_page (BoilerplatePreferencesView *self,
                                                const gchar                *name,
                                                GtkWidget                  *child)
{
    g_return_if_fail (BOILERPLATE_IS_PREFERENCES_VIEW (self));
    g_return_if_fail (name != NULL);
    g_return_if_fail (GTK_IS_WIDGET (child));

    GtkWidget *page = boilerplate_preferences_view_get_page_with_name (self, name);

    gtk_container_add (GTK_CONTAINER (page), GTK_WIDGET (child));
}

/**
 * boilerplate_preferences_view_get_page_with_name:
 * @object: An instance of #BoilerplatePreferencesView
 * @name: Id of the page for internal usage
 *
 * Gets the a #BoilerplatePreferencesPage with @name.
 *
 * Returns: (nullable): Returns a #GtkWidget of the page with @name.
 */
GtkWidget *
boilerplate_preferences_view_get_page_with_name (BoilerplatePreferencesView *self,
                                                 const gchar                *name)
{
    g_return_val_if_fail (BOILERPLATE_IS_PREFERENCES_VIEW (self), NULL);
    g_return_val_if_fail (name != NULL, NULL);

    BoilerplatePreferencesViewPrivate *priv = boilerplate_preferences_view_get_instance_private (self);

    GtkWidget *page = gtk_stack_get_child_by_name (GTK_STACK (priv->pages),
                                                   name);

    if (page == NULL)
    {
        return NULL;
    }

    return boilerplate_preferences_page_get_box (BOILERPLATE_PREFERENCES_PAGE (page));
}

/*
 * boilerplate_preferences_view_finalize:
 * @object: An instance of #BoilerplatePreferencesView
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
boilerplate_preferences_view_finalize (GObject *object)
{
    G_OBJECT_CLASS (boilerplate_preferences_view_parent_class)->finalize (object);
}

/*
 * boilerplate_preferences_view_init:
 * @self: An instance of #BoilerplatePreferencesView
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
boilerplate_preferences_view_init (BoilerplatePreferencesView *self)
{
    gtk_widget_init_template (GTK_WIDGET (self));
}

/*
 * boilerplate_preferences_view_class_init:
 * @klass: An instance of #BoilerplatePreferencesViewClass
 *
 * Initializes the various virtural functions in #BoilerplatePreferencesViewClass.
 * In addtition it connects the various #GtkWidgets from the gresource file
 * using gtk_widget_class_set_template_from_resource().
 */
static void
boilerplate_preferences_view_class_init (BoilerplatePreferencesViewClass *klass)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = boilerplate_preferences_view_finalize;

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/preferences/view.ui");

    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferencesView, box);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferencesView, sidebar);
    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferencesView, pages);
}

/**
 * boilerplate_preferences_view_new:
 *
 * Creates a new #BoilerplatePreferencesView, a sub-object of #GtkBin.
 *
 * ## Usage
 * |[
 * BoilerplatePreferencesView *preferences_view;
 *
 * preferences_view = boilerplate_preferences_view_new ();
 * ]|
 *
 * Returns: #BoilerplatePreferencesView; a sub-object of #GtkBin
 */
BoilerplatePreferencesView *
boilerplate_preferences_view_new (void)
{
    return g_object_new (BOILERPLATE_TYPE_PREFERENCES_VIEW,
                         NULL);
}

